import panflute as pf
import unittest

import linguafilter.gloss as gloss
import linguafilter.test_common as common

class TestParse(unittest.TestCase):
    def test_valid_content_strs_html(self):
        doc = common.MockDoc('html')
        span = pf.Span(pf.Str('a'), pf.Str('b'), pf.Str('c'), classes=['gloss'])
        gloss.parse(span, doc)
        self.assertEqual(pf.stringify(span), 'a<br/>b<br/>c')

    def test_valid_content_spans_html(self):
        doc = common.MockDoc('html')
        span1 = pf.Span(pf.Str('a'))
        span2 = pf.Span(pf.Str('b'))
        span3 = pf.Span(pf.Str('c'))
        span = pf.Span(span1, span2, span3, classes=['gloss'])
        gloss.parse(span, doc)
        self.assertEqual(pf.stringify(span), 'a<br/>b<br/>c')

    def test_valid_content_strs_and_spans_html(self):
        doc = common.MockDoc('html')
        span1 = pf.Span(pf.Str('a'))
        str1 = pf.Str('b')
        span2 = pf.Span(pf.Str('c'))
        span = pf.Span(span1, str1, span2, classes=['gloss'])
        gloss.parse(span, doc)
        self.assertEqual(pf.stringify(span), 'a<br/>b<br/>c')

    def test_valid_content_strs_latex(self):
        doc = common.MockDoc('latex')
        span = pf.Span(pf.Str('a'), pf.Str('b'), pf.Str('c'), classes=['gloss'])
        gloss.parse(span, doc)
        self.assertEqual(pf.stringify(span), '\\begin{exe}\n\\ex\n\\gll a\\\\\nb\\\\\n\\trans c\n\\end{exe}')

    def test_valid_content_spans_latex(self):
        doc = common.MockDoc('latex')
        span1 = pf.Span(pf.Str('a'))
        span2 = pf.Span(pf.Str('b'))
        span3 = pf.Span(pf.Str('c'))
        span = pf.Span(span1, span2, span3, classes=['gloss'])
        gloss.parse(span, doc)
        self.assertEqual(pf.stringify(span), '\\begin{exe}\n\\ex\n\\gll a\\\\\nb\\\\\n\\trans c\n\\end{exe}')

    def test_valid_content_strs_and_spans_latex(self):
        doc = common.MockDoc('latex')
        span1 = pf.Span(pf.Str('a'))
        str1 = pf.Str('b')
        span2 = pf.Span(pf.Str('c'))
        span = pf.Span(span1, str1, span2, classes=['gloss'])
        gloss.parse(span, doc)
        self.assertEqual(pf.stringify(span), '\\begin{exe}\n\\ex\n\\gll a\\\\\nb\\\\\n\\trans c\n\\end{exe}')

    def test_invalid_content(self):
        doc = common.MockDoc('html')

        span = pf.Span(classes=['gloss'])
        with self.assertRaisesRegexp(Exception, 'invalid gloss syntax'):
            gloss.parse(span, doc)

        span = pf.Span(pf.Str('a'), pf.Str('b'), classes=['gloss'])
        with self.assertRaisesRegexp(Exception, 'invalid gloss syntax'):
            gloss.parse(span, doc)

        span = pf.Span(pf.Str('a'), pf.Str('b'), pf.Str('c'), pf.Str('d'), classes=['gloss'])
        with self.assertRaisesRegexp(Exception, 'invalid gloss syntax'):
            gloss.parse(span, doc)



import string
import unittest

import linguafilter.ipa_mappings as mappings

class TestOrder(unittest.TestCase):
    def test_base_map_order(self):
        for key in mappings.BASE_MAP:
            self.assertEqual(key[1:], tuple(sorted(key[1:])))

    def test_mod_map_order(self):
        for key in mappings.MOD_MAP:
            self.assertEqual(key, tuple(sorted(key)))

class TestFormat(unittest.TestCase):
    def test_base_map_format(self):
        for key in mappings.BASE_MAP:
            val = mappings.BASE_MAP[key]

            # test html
            html = val['html']
            all(c in string.printable for c in html)
            self.assertTrue(len(html)==1 or len(html)==8 or len(html)==16)
            if len(html) != 1:
                self.assertEqual(html[:3], '&#x')
                int(html[3:7], 16) # raises if fails to parse
                self.assertEqual(html[7], ';')
                if len(html) == 16:
                    self.assertEqual(html[8:11], '&#x')
                    int(html[11:15], 16) # raises if fails to parse
                    self.assertEqual(html[15], ';')

            # test latex
            latex = val['latex']
            all(c in string.printable for c in latex)

    def test_mod_map_format(self):
        for key in mappings.MOD_MAP:
            val = mappings.MOD_MAP[key]

            # test html
            html = val['html']('a')
            all(c in string.printable for c in html)
            self.assertEqual(len(html), 9)
            self.assertEqual(html[0], 'a')
            self.assertEqual(html[1:4], '&#x')
            int(html[4:-1], 16) # raises if fails to parse
            self.assertEqual(html[8], ';')

            # test latex
            latex = val['latex']('a')
            all(c in string.printable for c in latex)



import panflute as pf

import linguafilter.common as common

def parse(elem, doc):
    if isinstance(elem, pf.Span) and 'phonfeat' in elem.classes:
        content = [c for c in elem.content if not isinstance(c, common.SPACE_TYPES)]

        elem.content = []
        if doc.format == 'html':
            elem.content.append(pf.RawInline('[', format=doc.format))
            for i in range(len(content)):
                elem.content.append(content[i])
                if i < len(content)-1:
                    elem.content.append(pf.RawInline(',', format=doc.format))
                    elem.content.append(pf.Space())
            elem.content.append(pf.RawInline(']', format=doc.format))
        elif doc.format == 'latex':
            elem.content.append(pf.RawInline('\\phonfeat{', format=doc.format))
            for i in range(len(content)):
                elem.content.append(content[i])
                if i < len(content)-1:
                    elem.content.append(pf.Space())
                    elem.content.append(pf.RawInline('\\\\', format=doc.format))
                    elem.content.append(pf.Space())
            elem.content.append(pf.RawInline('}', format=doc.format))



import panflute as pf

import linguafilter.common as common

def parse(elem, doc):
    if isinstance(elem, pf.Span) and 'gloss' in elem.classes:
        content = [c for c in elem.content if not isinstance(c, common.SPACE_TYPES)]
        if len(content) != 3:
            raise Exception('invalid gloss syntax')

        elem.content = []
        if doc.format == 'html':
            elem.content.append(content[0])
            elem.content.append(pf.RawInline('<br/>', format=doc.format))
            elem.content.append(content[1])
            elem.content.append(pf.RawInline('<br/>', format=doc.format))
            elem.content.append(content[2])
        elif doc.format == 'latex':
            elem.content.append(pf.RawInline('\\begin{exe}\n', format=doc.format))
            elem.content.append(pf.RawInline('\\ex\n', format=doc.format))
            elem.content.append(pf.RawInline('\\gll', format=doc.format))
            elem.content.append(pf.Space())
            elem.content.append(content[0])
            elem.content.append(pf.RawInline('\\\\\n', format=doc.format))
            elem.content.append(content[1])
            elem.content.append(pf.RawInline('\\\\\n', format=doc.format))
            elem.content.append(pf.RawInline('\\trans', format=doc.format))
            elem.content.append(pf.Space())
            elem.content.append(content[2])
            elem.content.append(pf.RawInline('\n', format=doc.format))
            elem.content.append(pf.RawInline('\\end{exe}', format=doc.format))



import panflute as pf
import unittest

import linguafilter.ipa as ipa
import linguafilter.test_common as common

Seg = ipa.Segment

class TestSegment(unittest.TestCase):
    def test_init(self):
        seg = Seg()
        self.assertIsNone(seg.base)
        self.assertEqual(seg.mods, [])

        seg = Seg('a')
        self.assertEqual(seg.base, 'a')
        self.assertEqual(seg.mods, [])

        seg = Seg('a', ['b'])
        self.assertEqual(seg.base, 'a')
        self.assertEqual(seg.mods, ['b'])

    def test_eq(self):
        seg1 = Seg('a', ['b'])
        seg2 = Seg('a', ['b'])
        seg3 = Seg('a', ['b', 'c'])
        seg4 = Seg('a', [])
        self.assertEqual(seg1, seg2)
        self.assertNotEqual(seg1, seg3)
        self.assertNotEqual(seg1, seg4)

    def test_str(self):
        self.assertEqual(str(Seg('a', ['b', 'c'])), "{ a, ['b', 'c'] }")

    def test_repr(self):
        self.assertEqual(repr(Seg('a', ['b', 'c'])), "{ a, ['b', 'c'] }")

class TestTokenize(unittest.TestCase):
    def test_simple_char(self):
        self.assertEqual(ipa.tokenize('a'), ['a'])
        self.assertEqual(ipa.tokenize('T'), ['T'])
        self.assertEqual(ipa.tokenize('6'), ['6'])
        self.assertEqual(ipa.tokenize('@'), ['@'])

    def test_simple_chars(self):
        actual = ipa.tokenize('abc9de0f@T6GMTqR')
        expected = ['a', 'b', 'c', '9', 'd', 'e', '0', 'f', '@', 'T', '6', 'G', 'M', 'T', 'q', 'R']
        self.assertEqual(actual, expected)

    def test_backslash_char(self):
        self.assertEqual(ipa.tokenize('r\\'), ['r\\'])
        self.assertEqual(ipa.tokenize('B\\'), ['B\\'])
        self.assertEqual(ipa.tokenize('3\\'), ['3\\'])
        self.assertEqual(ipa.tokenize('@\\'), ['@\\'])
        self.assertEqual(ipa.tokenize('<\\'), ['<\\'])
        self.assertEqual(ipa.tokenize('>\\'), ['>\\'])
        self.assertEqual(ipa.tokenize('=\\'), ['=\\'])
        self.assertEqual(ipa.tokenize('_\\'), ['_\\'])

    def test_slash_at_beginning_errors(self):
        with self.assertRaises(Exception):
            ipa.tokenize('\\')
        with self.assertRaises(Exception):
            ipa.tokenize('\\abcdef')

    def test_long_string(self):
        actual = ipa.tokenize('r\\bcde_\\_tgmr\\')
        expected = ['r\\', 'b', 'c', 'd', 'e', '_\\', '_', 't', 'g', 'm', 'r\\']
        self.assertEqual(actual, expected)

class TestProcessChars(unittest.TestCase):
    def test_simple_char(self):
        self.assertEqual(ipa.process_chars(['a']), [Seg('a')])
        self.assertEqual(ipa.process_chars(['T']), [Seg('T')])
        self.assertEqual(ipa.process_chars(['6']), [Seg('6')])
        self.assertEqual(ipa.process_chars(['@']), [Seg('@')])

    def test_single_chars_with_modifiers(self):
        self.assertEqual(ipa.process_chars(['a', '\'']), [Seg('a', ['\''])])
        self.assertEqual(ipa.process_chars(['a', '=']), [Seg('a', ['='])])
        self.assertEqual(ipa.process_chars(['a', '`']), [Seg('a', ['`'])])
        self.assertEqual(ipa.process_chars(['a', '~']), [Seg('a', ['~'])])

        self.assertEqual(ipa.process_chars(['a', '_', 'G']), [Seg('a', ['G'])])
        self.assertEqual(ipa.process_chars(['a', '_', '=']), [Seg('a', ['='])])
        self.assertEqual(ipa.process_chars(['a', '_', 'H', '_', 'T']), [Seg('a', ['H', 'T'])])

    def test_equivalent_chars(self):
        self.assertEqual(ipa.process_chars(['P']), [Seg('v\\')])
        self.assertEqual(ipa.process_chars(['v\\']), [Seg('v\\')])
        self.assertEqual(ipa.process_chars(['a', '_', 'j']), [Seg('a', ['\''])])
        self.assertEqual(ipa.process_chars(['a', '\'']), [Seg('a', ['\''])])

    def test_long_sequences(self):
        seq = ['r\\', '_', 'G', '\'', 'a', 'a', '~']
        expected = [Seg('r\\', ['G', '\'']), Seg('a'), Seg('a', ['~'])]
        self.assertEqual(ipa.process_chars(seq), expected)

    def test_mod_char_first(self):
        with self.assertRaisesRegexp(Exception, 'modifier characters must appear after a base character'):
            ipa.process_chars(['\''])
        with self.assertRaisesRegexp(Exception, 'modifier characters must appear after a base character'):
            ipa.process_chars(['='])
        with self.assertRaisesRegexp(Exception, 'modifier characters must appear after a base character'):
            ipa.process_chars(['`'])
        with self.assertRaisesRegexp(Exception, 'modifier characters must appear after a base character'):
            ipa.process_chars(['~'])

    def test_underscores_in_wrong_positions(self):
        with self.assertRaisesRegexp(Exception, 'underscore characters must appear after a base character'):
            ipa.process_chars(['_'])
        with self.assertRaisesRegexp(Exception, 'underscore characters must appear after a base character'):
            ipa.process_chars(['_', '_'])
        with self.assertRaisesRegexp(Exception, 'two underscore characters cannot appear next to each other'):
            ipa.process_chars(['a', '_', '_'])
        with self.assertRaisesRegexp(Exception, 'dangling modifier character'):
            ipa.process_chars(['a', '_'])
        with self.assertRaisesRegexp(Exception, 'two underscore characters cannot appear next to each other'):
            ipa.process_chars(['a', '_', '_', 'a'])
        with self.assertRaisesRegexp(Exception, 'underscore characters must appear after a base character'):
            ipa.process_chars(['_', '_', 'a'])

class TestRemoveMods(unittest.TestCase):
    def test_no_mods_nothing_to_remove(self):
        self.assertEqual(ipa.remove_mods([], []), [])

    def test_one_mod_nothing_to_remove(self):
        self.assertEqual(ipa.remove_mods(['='], []), ['='])

    def test_one_mod_one_mod_to_remove(self):
        self.assertEqual(ipa.remove_mods(['='], ['=']), [])

    def test_two_mods_one_mod_to_remove(self):
        self.assertEqual(ipa.remove_mods(['=', '`'], ['=']), ['`'])

class TestReduceMods(unittest.TestCase):
    def test_no_mods(self):
        self._assertModFuncsEqual(ipa.reduce_mods([]), [])

    def test_one_mod(self):
        self._assertModFuncsEqual(ipa.reduce_mods(['=']), ['a&#x0329;'])

    def test_two_mods(self):
        self._assertModFuncsEqual(ipa.reduce_mods(['=', 'd']), ['a&#x0329;', 'a&#x032A;'])

    def test_combining_mods(self):
        # TODO: once H_T is supported, uncomment
        # self._assertModFuncsEqual(ipa.reduce_mods(['H', 'T']), ['a&#x1DC4;'])
        pass

    def test_combining_mods_plus_others(self):
        # TODO: once H_T is supported, uncomment
        # self._assertModFuncsEqual(ipa.reduce_mods(['=', 'H', 'T']), ['a&#x1DC4;', 'a&#x0329;'])
        pass

    def _assertModFuncsEqual(self, funcs, results):
        self.assertEqual(len(funcs), len(results))
        for i in range(len(funcs)):
            self.assertEqual(funcs[i]('a'), results[i])

class TestReduceSeg(unittest.TestCase):
    def test_no_mods(self):
        self.assertEqual(ipa.reduce_seg(Seg('a')), ('a', []))

    def test_one_mod_no_reduction(self):
        self.assertEqual(ipa.reduce_seg(Seg('a', ['='])), ('a', ['=']))

    def test_one_mod_reduction(self):
        self.assertEqual(ipa.reduce_seg(Seg('r', ['`'])), ('&#x027D;', []))

    def test_multiple_mods_no_reduction(self):
        self.assertEqual(ipa.reduce_seg(Seg('r', ['=', 'G'])), ('r', ['=', 'G']))

    def test_multiple_mods_reduction(self):
        self.assertEqual(ipa.reduce_seg(Seg('r', ['=', '`'])), ('&#x027D;', ['=']))
        # no actual cases where two mods reduce into the base yet

    def test_no_translation(self):
        self.assertEqual(ipa.reduce_seg(Seg('#')), ('#', []))
        self.assertEqual(ipa.reduce_seg(Seg('(')), ('(', []))
        self.assertEqual(ipa.reduce_seg(Seg(')')), (')', []))

    def test_no_translation_with_mods(self):
        self.assertEqual(ipa.reduce_seg(Seg('#', ['='])), ('#', ['=']))

class TestStringify(unittest.TestCase):
    def test_multiple_mods(self):
        self.assertEqual(ipa.stringify(Seg('r', ['`', 'G', 'A'])), '&#x027D;&#x02E0;&#x0318;')
    # TODO: once H_T etc. are supported, include these in the tests

class TestParse(unittest.TestCase):
    def test_no_str(self):
        doc = common.MockDoc('html')
        span = pf.Span(classes=['ipa'])
        ipa.parse(span, doc)
        self.assertEqual(pf.stringify(span), '')

    def test_str(self):
        doc = common.MockDoc('html')
        span = pf.Span(pf.Str('A'), classes=['ipa'])
        ipa.parse(span, doc)
        self.assertEqual(pf.stringify(span), '&#x0251;')

    def test_latex(self):
        doc = common.MockDoc('latex')
        span = pf.Span(pf.Str('A'), classes=['ipa'])
        ipa.parse(span, doc)
        self.assertEqual(pf.stringify(span), '\\textipa{A}')

    def test_latex_with_space(self):
        doc = common.MockDoc('latex')
        span = pf.Span(pf.Str('A'), pf.Space(), pf.Str('B'), classes=['ipa'])
        ipa.parse(span, doc)
        self.assertEqual(pf.stringify(span), '\\textipa{A} \\textipa{B}')

class TestGetCombos(unittest.TestCase):
    def test_single_element(self):
        self.assertEqual(ipa.get_combos([1]), [[1], []])

    def test_two_elements(self):
        self.assertEqual(ipa.get_combos([1, 2]), [[1, 2], [1], [2], []])

    def test_three_elements(self):
        self.assertEqual(ipa.get_combos([1, 2, 3]), [[1,2,3],[1,2],[1,3],[2,3],[1],[2],[3],[]])



import os.path
import panflute as pf
import unittest

import linguafilter.lexicon as lexicon
import linguafilter.test_common as common

class TestGetConfig(unittest.TestCase):
    def test_defaults(self):
        config = lexicon.get_config({'file': 'file.csv'})
        self.assertEqual(config, {'file': 'file.csv', 'delimiter': ',', 'quote': '"'})

    def test_no_file(self):
        with self.assertRaisesRegexp(Exception, 'filename required'):
            lexicon.get_config({})

    def test_tab_delim(self):
        config = lexicon.get_config({'file': 'file.csv', 'delim': '\\t'})
        self.assertEqual(config, {'file': 'file.csv', 'delimiter': '\t', 'quote': '"'})

    def test_all_specified(self):
        config = lexicon.get_config({'file': 'file.csv', 'delim': '\t', 'quote': '|'})
        self.assertEqual(config, {'file': 'file.csv', 'delimiter': '\t', 'quote': '|'})

class TestReplaceFields(unittest.TestCase):
    def test_no_replacements(self):
        s = pf.Str('abcd')
        lexicon.replace_fields({}, s)
        self.assertEqual(s.text, 'abcd')

    def test_not_str(self):
        span = pf.Span(pf.Str('{field}'))
        lexicon.replace_fields({'field': 'value'}, span)
        self.assertEqual(span.content[0].text, '{field}')

    def test_one_replacement(self):
        s = pf.Str('{field}')
        lexicon.replace_fields({'field': 'value'}, s)
        self.assertEqual(s.text, 'value')

    def test_repeated_replacements(self):
        s = pf.Str('{field1}{field2}{field3}{field2}')
        lexicon.replace_fields({'field1': 'value1', 'field2': 'value2', 'field3': 'value3'}, s)
        self.assertEqual(s.text, 'value1value2value3value2')

    def test_unused_replacements(self):
        s = pf.Str('{field1}{field2}')
        lexicon.replace_fields({'field1': 'value1', 'field2': 'value2', 'field3': 'value3'}, s)
        self.assertEqual(s.text, 'value1value2')

    def test_nonexistant_replacements(self):
        s = pf.Str('{field1}{field2}{field3}')
        lexicon.replace_fields({'field1': 'value1', 'field3': 'value3'}, s)
        self.assertEqual(s.text, 'value1{field2}value3')

class TestParse(unittest.TestCase):
    def test_empty(self):
        doc = common.MockDoc('html')
        filename = os.path.join('linguafilter', 'test_lexicon.csv')
        div = pf.Div(attributes={'file': filename}, classes=['lexicon'])
        lexicon.parse(div, doc)
        self.assertEqual(pf.stringify(div), '')

    def test_single_line(self):
        doc = common.MockDoc('html')
        filename = os.path.join('linguafilter', 'test_lexicon.csv')
        attributes = {'file': filename}
        div = pf.Div(pf.Para(pf.Str('{field1}, {field2}, and {field3}')), attributes=attributes, classes=['lexicon'])
        lexicon.parse(div, doc)
        self.assertEqual(pf.stringify(div), 'r1f1, r1f2, and r1f3\n\nr2f1, r2f2, a field, and r2f3\n\nr3f1, r3f2, and r3f3\n\n')



import panflute as pf

import linguafilter.lexicon as lexicon
import linguafilter.gloss as gloss
import linguafilter.ipa as ipa
import linguafilter.phonfeat as phonfeat
import linguafilter.phonrule as phonrule

def main():
    pf.run_filters([lexicon.parse, gloss.parse, ipa.parse, phonfeat.parse, phonrule.parse])

if __name__ == '__main__':
    main()



import panflute as pf
import unittest

import linguafilter.phonrule as phonrule
import linguafilter.test_common as common

class TestParse(unittest.TestCase):
    def test_valid_content_strs(self):
        doc = common.MockDoc('html')
        span = pf.Span(pf.Str('a'), pf.Str('b'), pf.Str('c'), pf.Str('d'), classes=['phonrule'])
        phonrule.parse(span, doc)
        self.assertEqual(pf.stringify(span), 'a -&gt; b/c_d')

    def test_valid_content_spans(self):
        doc = common.MockDoc('html')
        span1 = pf.Span(pf.Str('a'))
        span2 = pf.Span(pf.Str('b'))
        span3 = pf.Span(pf.Str('c'))
        span4 = pf.Span(pf.Str('d'))
        span = pf.Span(span1, span2, span3, span4, classes=['phonrule'])
        phonrule.parse(span, doc)
        self.assertEqual(pf.stringify(span), 'a -&gt; b/c_d')

    def test_valid_content_strs_and_spans(self):
        doc = common.MockDoc('html')
        span1 = pf.Span(pf.Str('a'))
        str1 = pf.Str('b')
        span2 = pf.Span(pf.Str('c'))
        str2 = pf.Str('d')
        span = pf.Span(span1, str1, span2, str2, classes=['phonrule'])
        phonrule.parse(span, doc)
        self.assertEqual(pf.stringify(span), 'a -&gt; b/c_d')

    def test_valid_content_strs_and_spans_latex(self):
        doc = common.MockDoc('latex')
        span1 = pf.Span(pf.Str('a'))
        str1 = pf.Str('b')
        span2 = pf.Span(pf.Str('c'))
        str2 = pf.Str('d')
        span = pf.Span(span1, str1, span2, str2, classes=['phonrule'])
        phonrule.parse(span, doc)
        self.assertEqual(pf.stringify(span), '\\phonb{a}{b}{c}{d}')

    def test_invalid_content(self):
        doc = common.MockDoc('html')

        span = pf.Span(classes=['phonrule'])
        with self.assertRaisesRegexp(Exception, 'invalid phonrule syntax'):
            phonrule.parse(span, doc)

        span = pf.Span(pf.Str('a'), pf.Str('b'), classes=['phonrule'])
        with self.assertRaisesRegexp(Exception, 'invalid phonrule syntax'):
            phonrule.parse(span, doc)

        span = pf.Span(pf.Str('a'), pf.Str('b'), pf.Str('c'), pf.Str('d'), pf.Str('e'), classes=['phonrule'])
        with self.assertRaisesRegexp(Exception, 'invalid phonrule syntax'):
            phonrule.parse(span, doc)



import copy
import csv
import io
import panflute as pf

import linguafilter.common as common

def get_config(attrs):
    config = {}
    if 'file' not in attrs:
        raise Exception('filename required')
    config['file'] = attrs['file']
    if 'delim' in attrs:
        config['delimiter'] = '\t' if attrs['delim']=='\\t' else attrs['delim']
    else:
        config['delimiter'] = ','
    config['quote'] = attrs['quote'] if 'quote' in attrs else '"'
    return config

def replace_fields(fields, elem):
    if isinstance(elem, pf.Str):
        for field in fields:
            elem.text = elem.text.replace('{'+field+'}', fields[field])

def parse(elem, doc):
    if isinstance(elem, pf.Div) and 'lexicon' in elem.classes:
        config = get_config(elem.attributes)
        content = elem.content
        elem.content = []
        with io.open(config['file'], newline='') as f:
            reader = csv.DictReader(f, delimiter=config['delimiter'], quotechar=config['quote'])
            for row in reader:
                for c in content:
                    # don't copy parent, because otherwise it will copy the
                    # entire document or some nonsense
                    old_parent = c.parent
                    c.parent = None
                    new_c = copy.deepcopy(c)
                    c.parent = old_parent
                    new_c.parent = old_parent

                    func = lambda elem, doc: replace_fields(row, elem)
                    new_c.walk(func)
                    elem.content.append(new_c)
